local actions = require("telescope.actions")
local previewers = require("telescope.previewers")
local builtin = require("telescope.builtin")

require("telescope").setup({
	defaults = {
		dynamic_preview_title = true,
		vimgrep_arguments = {
			"rg",
			"--color=never",
			"--no-heading",
			"--with-filename",
			"--line-number",
			"--column",
			"--smart-case",
			"--trim",
		},
		preview = {
			filesize_limit = 5,
			timeout = 150,
			treesitter = true,
		},
		borderchars = { " ", " ", " ", " ", " ", " ", " ", " " },
		layout_config = {
			prompt_position = "top",
		},
		prompt_prefix = "     ",
		selection_caret = " → ",
		entry_prefix = "  ",
		file_ignore_patterns = { "node_modules", ".git/", "dist/" },
		set_env = { ["COLORTERM"] = "truecolor" },
		mappings = {
			i = {
				["<C-x>"] = false,
				["<C-o>"] = actions.select_horizontal,
				["<CR>"] = actions.select_default + actions.center,
				["<esc>"] = actions.close,
				["<TAB>"] = actions.toggle_selection + actions.move_selection_next,
			},
		},
	},
	pickers = {
		buffers = {
			theme = "dropdown",
			results_height = 20,
			initial_mode = "normal",
			borderchars = { " ", " ", " ", " ", " ", " ", " ", " " },
			previewer = false,
			disable_devicons = true,
			mappings = {
				n = {
					["d"] = actions.delete_buffer,
				},
			},
			sort_mru = true,
			sort_lastused = true,
			ignore_current_buffer = true,
		},
		find_files = {
			find_command = { "fd", "--type", "f", "-c always", "-H", "--strip-cwd-prefix" },
			hidden = true,
		},
	},
	extensions = {
		file_browser = {
			hidden = true,
			hijack_netrw = true,
			theme = "ivy",
			borderchars = { " ", " ", " ", " ", " ", " ", " ", " " },
			grouped = true,
			initial_mode = "normal",
			sorting_strategy = "ascending",
		},
	},
})

local delta_bcommits = previewers.new_termopen_previewer({
	get_command = function(entry)
		return {
			"git",
			"-c",
			"core.pager=delta",
			"-c",
			"delta.side-by-side=false",
			"diff",
			entry.value .. "^!",
			"--",
			entry.current_file,
		}
	end,
})

local delta = previewers.new_termopen_previewer({
	get_command = function(entry)
		return { "git", "-c", "core.pager=delta", "-c", "delta.side-by-side=false", "diff", entry.value .. "^!" }
	end,
})

Delta_git_commits = function(opts)
	opts = opts or {}
	opts.previewer = {
		delta,
		previewers.git_commit_message.new(opts),
		previewers.git_commit_diff_as_was.new(opts),
	}
	builtin.git_commits(opts)
end

Delta_git_bcommits = function(opts)
	opts = opts or {}
	opts.previewer = {
		delta_bcommits,
		previewers.git_commit_message.new(opts),
		previewers.git_commit_diff_as_was.new(opts),
	}
	builtin.git_bcommits(opts)
end

Project_files = function()
	local opts = { show_untracked = false } -- define here if you want to define something
	local ok = pcall(builtin.git_files, opts)
	if not ok then
		builtin.find_files(opts)
	end
end

local default_opts = { noremap = true, silent = true }
vim.api.nvim_set_keymap("n", "<F1>", "<cmd>Telescope help_tags<CR>", default_opts)
vim.api.nvim_set_keymap("n", "<F2>", "<cmd>lua Project_files()<CR>", default_opts)
vim.api.nvim_set_keymap(
	"n",
	"<F3>",
	"<cmd>lua require'telescope'.extensions.file_browser.file_browser({path = vim.fn.expand('%:p:h')})<CR>",
	default_opts
)
vim.api.nvim_set_keymap("n", "<F4>", "<cmd>Telescope buffers<CR>", default_opts)
vim.api.nvim_set_keymap("n", "<F5>", "<cmd>lua Delta_git_commits()<CR>", default_opts)
vim.api.nvim_set_keymap("n", "<F6>", "<cmd>lua Delta_git_bcommits()<CR>", default_opts)
vim.api.nvim_set_keymap("n", "<M-f>", "<cmd>Telescope live_grep<CR>", default_opts)
vim.api.nvim_set_keymap("n", "<M-S-r>", "<cmd>Telescope resume<CR>", default_opts)
require("telescope").load_extension("fzf")
require("telescope").load_extension("file_browser")
